DEBUG = True
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'django_d2m.db',
    }
}

MIDDLEWARE_CLASSES = (
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
)

STATIC_URL = '/'
SECRET_KEY = 'django_d2m'

INSTALLED_APPS = (
    'django.contrib.staticfiles',
    'django_d2m.testapp',
)



from django_d2m.testapp.models import Author, Tag, Article, Country
from django_d2m import dict_to_model, list_to_model, queryset_to_model
from django.db.models import Sum, Count
from django.test import TestCase


class D2MTestCase(TestCase):
    def assertDictList(self, l1, l2, order_by=None):
        if order_by:
            from operator import itemgetter
            l1, l2 = [sorted(_, key=itemgetter(order_by))
                      for _ in (l1, l2)]

        pairs = zip(l1, l2)
        self.assertFalse(any(x != y for x, y in pairs))

    def test_base(self):
        self.assertTrue(True)

    def test_dict_to_model(self):
        author_lqez = Author.objects.get(name='lqez')

        result = dict_to_model(
            Article.objects.filter(author=author_lqez)
            .values('author').annotate(total_read=Sum('read'))[0],
            Article
        )

        self.assertEqual(result['total_read'], 200)

    def test_list_to_model(self):
        result = list_to_model(
            list(Article.objects.values('author')
                 .annotate(total_read=Sum('read'))),
            Article
        )

        expected = [
            {'total_read': 200,
             'author': Author.objects.get(name='lqez')},
            {'total_read': 1000,
             'author': Author.objects.get(name='Prodo')},
            {'total_read': 30000,
             'author': Author.objects.get(name='Tolkien')},
        ]

        self.assertDictList(result, expected, 'total_read')

    def test_queryset_to_model(self):
        result = queryset_to_model(
            Article.objects.values('tags')
            .annotate(count=Count('tags'))
        )

        expected = [
            {'count': 3,
             'tags': Tag.objects.get(name='Diary')},
            {'count': 5,
             'tags': Tag.objects.get(name='Fiction')},
            {'count': 1,
             'tags': Tag.objects.get(name='IT')},
        ]

        self.assertDictList(result, expected, 'count')

    def test_double(self):
        result = queryset_to_model(
            Article.objects.values('author__country')
            .annotate(total_read=Sum('read'))
        )

        expected = [
            {'total_read': 200,
             'author__country': Country.objects.get(name='Korea')},
            {'total_read': 31000,
             'author__country': Country.objects.get(name='Endor')},
        ]

        self.assertDictList(result, expected, 'total_read')



version_info = (0, 1, 1)

__version__ = VERSION = '.'.join(map(str, version_info))
__project__ = PROJECT = 'django-d2m'

from django.db.models import ForeignKey
from django.db.models.fields import FieldDoesNotExist
from collections import defaultdict

__all__ = ['dictlist_to_model', ]


def _get_field_class_matching(base_model, d):
    class NotForeignKey(Exception):
        pass

    result = {}

    for k in d.keys():
        model = base_model
        try:
            for field in k.split('__'):
                field_object, _, _, m2m = \
                    model._meta.get_field_by_name(field)
                if m2m or isinstance(field_object, ForeignKey):
                    model = field_object.rel.to
                else:
                    raise NotForeignKey

            result[k] = model
        except (FieldDoesNotExist, NotForeignKey):
            continue

    return result


def queryset_to_model(qs, basemodel=None):
    # Convert queryset to list
    if isinstance(qs, list):
        qs = list(qs)

    # Raise when None
    if len(qs) == 0:
        raise ValueError

    # Get field matching table with first one
    matches = _get_field_class_matching(
        basemodel if basemodel else qs.model,
        qs[0]
    )

    # Pull out candidates from ids
    candidates = defaultdict(dict)
    for k, model in matches.iteritems():
        for obj in model.objects.filter(id__in=set([_[k] for _ in qs])):
            candidates[k][obj.id] = obj

    # Convert result with candidates
    result = []
    for item in qs:
        for k, v in candidates.iteritems():
            item[k] = v[item[k]]
        result.append(item)

    return result


def list_to_model(l, basemodel):
    return queryset_to_model(l, basemodel)


def dict_to_model(d, basemodel):
    # Get field matching table with first one
    matches = _get_field_class_matching(basemodel, d)

    # Convert
    for k, model in matches.iteritems():
        d[k] = model.objects.get(id=d[k])

    return d



from setuptools import setup, find_packages
from django_d2m import VERSION, PROJECT


CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Utilities',
]


setup(
    name=PROJECT,
    version=VERSION,
    packages=find_packages(),
    zip_safe=True,

    author='Park Hyunwoo',
    author_email='ez.amiryo' '@' 'gmail.com',
    maintainer='Park Hyunwoo',
    maintainer_email='ez.amiryo' '@' 'gmail.com',
    url='https://github.com/lqez/django-d2m',

    description='Mapping annotated dict list into Django models',
    classifiers=CLASSIFIERS,

    install_requires=['django'],
    test_suite='runtests.runtests',
)



from django.db import models


class Country(models.Model):
    name = models.CharField(max_length=100)

    def __unicode__(self):
        return self.name


class Author(models.Model):
    name = models.CharField(max_length=100)
    country = models.ForeignKey(Country)

    def __unicode__(self):
        return self.name


class Tag(models.Model):
    name = models.CharField(max_length=100)

    def __unicode__(self):
        return self.name


class Article(models.Model):
    title = models.CharField(max_length=100)
    read = models.IntegerField(default=0)

    author = models.ForeignKey(Author)
    tags = models.ManyToManyField(Tag)

    def __unicode__(self):
        return self.title
